/*
 Copyright (C) 2024 3NSoft Inc.

 This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

import { userSystem } from "@/services";
import { useAppStore, useProcessStore } from "@/store";
import { ProcessStore } from "@/store/process/types";
import { AppInfo } from "@/types";
import { compare as compareSemVer } from 'semver';

/**
 * This "controller" function checks all installed apps for updates.
 * It updates apps' info in AppStore.
 */
export async function checkForAllUpdates(
  forceInfoDownload = false
): Promise<void> {

  const appStore = useAppStore();
  const processStore = useProcessStore();

  const installedApps = appStore.applicationsInSystem
  .filter(({ versions: { current } }) => !!current);
  for (const app of installedApps) {
    await checkAppUpdates(app, processStore, forceInfoDownload);
  }

  await checkPlatformUpdates(processStore, forceInfoDownload);

}

async function checkAppUpdates(
  app: AppInfo, processStore: ProcessStore, forceInfoDownload: boolean
): Promise<void> {
  const { appId, versions: { latest: latestInSys } } = app;
  processStore.upsertProcess(appId, {
    procType: 'update-checking'
  });
  try {

    const distInfo = await userSystem.getAppDistInfo(appId, forceInfoDownload);
    if (!distInfo?.versions) {
      app.availableUpdates = undefined;
      return;
    }

    for (const [ channel, version ] of Object.entries(distInfo.versions)) {
      if (compareSemVer(version, latestInSys) > 0) {
        if (!app.availableUpdates) {
          app.availableUpdates = [];
        }
        app.availableUpdates.push({
          channel, version
        });
      }
    }

  } finally {
    processStore.delProcess(appId, 'update-checking');
  }
}

async function checkPlatformUpdates(
  processStore: ProcessStore, forceInfoDownload: boolean
): Promise<void> {

  console.log(`🧐 missing implementation to check platform's update`);

}
